/*
 * Listenoperationen
 * Implementation der Listenoperationen aus liste.h
 */

# include <stdio.h>
# include <math.h>
# include <stdlib.h> 
# include "liste.h"

# define M_PI 3.14159265359

/*
 * Koordinatenumrechnung
 */
struct kartes kartesisch( struct polar p)
{ struct kartes k;
  p.phi *= M_PI / 180;   /* Winkelmass --> Bogenmass */

  k.x = p.rho * cos( p.phi);           /* Umrechnung */
  k.y = p.rho * sin( p.phi);
  return k;
}

/*
 * Sortierfunktion
 * von vorn nach hinten
 */
void sortier_ein( struct Punkt **a, struct Punkt p)
{
  while( *a && ( (*a) -> kar.x < p.kar.x ||
                 (*a) -> kar.x == p.kar.x &&
                 (*a) -> kar.y < p.kar.y))
    a = &( *a) -> Nf;
  p.Nf = *a;
  *a = (struct Punkt *) malloc( sizeof( struct Punkt));
  **a = p;
  return;
}


/*
 * Ausgabe eines Punktes
 */
void printPunkt( struct Punkt p)
{
  printf( "(%12f,%12f)  ", p.pol.rho, p.pol.phi);
  printf( "(%12f,%12f)\n", p.kar.x, p.kar.y);
}


/*
 * Ausgabe einer Liste
 */
void printListe( struct Punkt *a)
{
  printf( "Liste sortiert (polar/kartesisch):\n");
  while( a)
  { printPunkt( *a);
    a = a -> Nf;
  }
}


/*
 * Punkt suchen
 * Suche nach Punkt mit kartesischen Koordinaten
 */
struct Punkt *kar_suchen
  ( struct Punkt *a, struct Punkt p)
{
  while( a && (  a->kar.x < p.kar.x || 
                 a -> kar.x == p.kar.x &&
                 a -> kar.y < p.kar.y)) a = a->Nf;
  if( a && a -> kar.x == p.kar.x &&
           a -> kar.y == p.kar.y) return a;
  return NULL;
}


/*
 * Punkt suchen
 * Suche nach Punkt mit Polarkoordinaten
 */
struct Punkt *pol_suchen
  ( struct Punkt *a, struct Punkt p)
{
  p.kar = kartesisch( p.pol);
  return( kar_suchen( a, p));
}


/*
 * Punkt loeschen
 * Entfernen mittels kartesischen Koordinaten
 * Verfahren wie bei kar_suchen
 */
void kar_loeschen( struct Punkt **a, struct Punkt p)
{
  struct Punkt **v;
       /* Zeiger auf Adresse des vorherigen Elements */
  v = a;
  while ( *a && ( (*a) -> kar.x < p.kar.x || 
                  (*a) -> kar.x == p.kar.x &&

                  (*a) -> kar.y < p.kar.y))
  { v = a; a = &(*a) ->Nf;
  }
  if ( *a && (*a) -> kar.x == p.kar.x &&
             (*a) -> kar.y == p.kar.y) 
  { if(v != a)
      (*v) -> Nf = (*a) -> Nf;
    else
      *v = (*a) -> Nf;             /* erstes Element */
    free( *a);
  }
  return;
}

/*
 * Punkt loeschen
 * Entfernen mittels Polarkoordinaten
 */
void pol_loeschen
  ( struct Punkt **liste, struct Punkt p)
{
  p.kar = kartesisch( p.pol);
  kar_loeschen( liste, p); 
}

/*
 * Liste umkehren
 * Umkehrfunktion 1: Richtungsaenderung der Zeiger
 */
void inverse1( struct Punkt **a)
{ struct Punkt *z1, *z2 = NULL;
  if( ! *a) return;                   /* leere Liste */
  while( (*a) -> Nf)
  { z1 = (*a) -> Nf;     /* z1 auf naechstes Element */
    (*a) -> Nf = z2;    /* Nf auf vorheriges Element */

    z2 = *a;             /* z2 auf aktuelles Element */
    *a = z1;          /* naechstes Element aktuelles */
  }
  (*a) -> Nf = z2;      /* Nf auf vorheriges Element */

  return;
}


/*
 * Liste umkehren
 * Umkehrfunktion 2  mittels zweier Hilfsfunktionen
 */
void inverse2( struct Punkt **a)
{
  struct Punkt *inv_a = NULL, *p;
  while( *a)            /* Liste ist noch nicht leer */
  {
    p = haenge_ab( a); haenge_an( &inv_a, p);
  }
  *a = inv_a;  /* Listenadresse auf umgekehrte Liste */
  return;
}


/*
 * Hilfsfunktionen
 * Funktion nimmt das obere Element heraus
 */

struct Punkt *haenge_ab( struct Punkt **a)

{ struct Punkt *p;
  p = *a;    /* oberstes Element gelesen, abgehaengt */
  *a = (*a) -> Nf;
  return p;
}


/*
 * Funktion legt ein oberes Element hinein
 */
void haenge_an( struct Punkt **a, struct Punkt *p)
{
  p -> Nf = *a;    /* Anhaengen der Liste an Element */
  *a = p;                /* Listenanfang auf Element */
  return;
}
